package Scorch;

// CLASS: Physics
// AUTHOR: Ramya Ramesh
// PURPOSE: This class manages the estimation of the projectile
// trajectories, Winds, and Hazard conditions.

import java.awt.*;

// CLASS Physics definitions
public class Physics
{
    /////////////////////////////////
    // Constants
    // No wind indicator
    public final int NO_WIND = 0; // Constant wind indicator 
    public final int CONSTANT_WIND = 1; // Variable wind velocity indicator
    public final int CHANGING_WIND = 2; 
 
    // Default Earths gravity
    final static double EARTH_GRAVITY = 9.8;
    /////////////////////////////////

    /////////////////////////////////
    // Class member variables 
    
    // Gravity value
    static private double Gravity = EARTH_GRAVITY;
    // Wind condition
    static private int Wind;
    // Constant Wind Velocity
    static private int ConstantWindVelocity;
    // Variable Wind Velocity
    static private int VariableWindVelocity;
    
    // Wind flag (enabled/disabled) 
    private boolean WindFlag;
    // Step Size
    private double StepSize;
    
    // Angle and speed of projectile launch
    private int Angle;
    private double Speed, VelX0, VelY0;
    // Starting X and Y locations of projectile
    private int startX,startY;
    // Current X and Y positions
    private int X, Y;
    // Current step in trajectory
    private int StepN;
    ///////////////////////////////////
    
    
    //////////////////////////////////// 
    // Methods of Physics class
    
    // Constructor
    public Physics(int x0, int y0, int angl, double power)
    {
	Angle = angl; // Set angle of launch
	Speed = power; // set initial speed
	startX = x0; // set start X location
	startY = y0; // set start Y location
	
	X = x0;
	Y = y0;
	
	// Horizontal and vertical components of velocity
	double angle = (double) Angle*Math.PI/180.00;

	VelX0 = Speed*Math.cos(angle);
	VelY0 = Speed*Math.sin(angle);
       
	// Set StepSize
	/*StepSize = Math.abs(4/VelX0) * Math.sqrt(Math.abs(VelX0)/62.0);
	
	if (Angle == 90 || Speed == 0)
	    StepSize = 0.08;
	else
	    if (Angle > 80 && Angle < 100)
		StepSize = StepSize * 
		    (((Math.abs(90.0 - (double)Angle)-5.0)/2.0)+5) / 10.0;
	*/
	StepSize = 0.1;

	// Initialize Step to 0
	StepN=0;
	
	WindFlag = false; // Default disable wind effects
	Wind = NO_WIND; // No wind effects
    }
    
    // Set gravity to arbitrary value
    public static void setGravity(double grav)
    {
	Gravity = grav;  
    }

    public Point getPosition()
    {
        return new Point(startX,startY);
    }

    public int getStartX()
    {
	return startX;
    }

    public int getStartY()
    {
	return startY;
    }

    public int getAngle()
    {
        return Angle;
    }            

    // Set wind to one of three values: NO_WIND, CONSTANT_WIND or
    // CHANGING_WIND
    public static void setWind(int NewWind)
    {
	ConstantWindVelocity = NewWind;
    }

    public static int getWind()
    {
	return ConstantWindVelocity;
    }


    // Calculate trajectory for specified Steps.
    // Assumes that the vectors Xpos[] and Ypos[] have already been allocated
    // Note StepSize determines jump size and may need to be set properly
    public void CalcTrajectory(int Xpos[],int Ypos[],int steps)
    {
	// ERROR Condition if steps is <= 0
	if (steps<=0) 
	    return;
	
	double deltax, deltay;
	
	// Iterate through each step
	for(int ii=StepN;ii<StepN+steps;++ii)
	    {
		
		// Compute X and Y incremenets according to equations
		deltax = (ConstantWindVelocity+VelX0)*ii*StepSize;
		deltay = ii*StepSize*(VelY0 - 0.5*Gravity*ii*StepSize);
		
		Xpos[ii-StepN] = (int) (startX + deltax);
		Ypos[ii-StepN] = (int) (startY + deltay);
	    }
	// Update the StepN position
	StepN += steps;
	
	// Store current location
	X = Xpos[steps-1];
	Y = Ypos[steps-1];
	
    }

    public double getHSpeed()
    {
	return VelX0;
    }

    public String toString()
    {
	return "x: "+X+" y: "+Y+" power: "+(Speed*8)+" angle: "+Angle;
    }
}
