package Scorch.gui;

/*
  Class:  ShopWindow
  Author: Mikhail Kruk
  Desciption: the ScorchShop window which allows users to buy items and 
  weapons for the next rounds
*/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import Scorch.*;
import Scorch.Weapons.*;
import Scorch.Items.*;
import swindows.*;

public class ShopWindow extends sWindow implements ActionListener,
						   KeyListener
{
    private ScrollPane scrollPane;
    private Button finish, cancel, leave;
    private ScorchPlayer myPlayer;
    private long cash;
    private Label lCash;
    private Vector shopItems;

    public ShopWindow(int w, int h, ScorchApplet owner)
    {
	super(0,0,w,h,"Scorched Earth Shop", owner);
   
	myPlayer = owner.getMyPlayer();
	Vector weapons = myPlayer.getWeapons();
	Vector items = myPlayer.getItems();
	int count = 0;
	
	if( weapons != null ) count += weapons.size();
	if( items != null ) count += items.size();

	cash = myPlayer.getCash();

	Panel itemsPanel = new Panel(new GridLayout(count,1,0,4)),
	    mainPanel = new Panel(new BorderLayout(0,0));
		
	shopItems = new Vector();
	
	loadShopItems(itemsPanel, weapons);
	loadShopItems(itemsPanel, items);

	Panel legend = new Panel(new GridLayout(1,5));

	legend.add(new Label("item", Label.CENTER));
	legend.add(new Label("price", Label.CENTER));
	legend.add(new Label("amount", Label.CENTER));
	legend.add(new Label("order", Label.CENTER));
	legend.add(new Label(""));

	scrollPane = new ScrollPane();
	
	mainPanel.add(legend, BorderLayout.NORTH);
	mainPanel.add(itemsPanel, BorderLayout.CENTER);
	
	scrollPane.add(mainPanel);

	finish = new Button("Confirm order");
	cancel = new Button("Cancel order");
	leave = new Button("Leave the game");

	finish.addActionListener(this);
	cancel.addActionListener(this);
	leave.addActionListener(this);

	Panel bp = new Panel();
	bp.add(finish);
	bp.add(cancel);
	bp.add(leave);

	lCash = new Label("", Label.CENTER);
	updateCash(0);

	setLayout(new BorderLayout(0,0));
	add(lCash, BorderLayout.NORTH);
	add(scrollPane, BorderLayout.CENTER);
	add(bp, BorderLayout.SOUTH);

	addKeyListener(this);
    }
    
    // load shopItems panels corresponding to the items vector
    // into to the Panel pItems and also make a copy [reference] to them
    // from the list shopItems
    private void loadShopItems(Panel pItems, Vector items)
    {
	for(int i = 0; items != null && i < items.size(); i++)
	    {
		ShopItem si = 
		    new ShopItem(this, ((Item)items.elementAt(i)));
		pItems.add(si);
		shopItems.addElement(si);
	    }
    }
    
    // change amount of cash and update the label in the window
    public void updateCash(long inc)
    {
	cash += inc;
	lCash.setText("Please buy weapons and items for the next round. "+
		      "You have $"+cash+" left");

	for(int i = 0; i < shopItems.size(); i++)
	    ((ShopItem)shopItems.elementAt(i)).updateButtons();
    }

    public void display()
    {
	super.display();
	center();
    }

    public void actionPerformed(ActionEvent evt)
    {
	Object source = evt.getSource();

	if( source == finish )
	    {
		confirm();
		return;
	    }
	if( source == cancel )
	    {
		close();
		((ScorchApplet)owner).startGame();
	    }
	if( source == leave )
	    {
		close();
		((ScorchApplet)owner).Quit();
		return;
	    }
    }

    private void confirm()
    {
	close();
	for(int i = 0; i < shopItems.size(); i++)
	    ((ShopItem)shopItems.elementAt(i)).confirmOrder();
	myPlayer.setCash(cash);
	((ScorchApplet)owner).startGame();
    }

    public void keyPressed(KeyEvent evt)
    {
	if( evt.getKeyCode() == KeyEvent.VK_ESCAPE )
	    {
		close();
		((ScorchApplet)owner).startGame();
	    }

	if( evt.getKeyCode() == KeyEvent.VK_ENTER )
	    {
		confirm();
		((ScorchApplet)owner).startGame();
	    }
    }

    public void keyReleased(KeyEvent evt) {}
    public void keyTyped(KeyEvent evt) {}
    
    public boolean hasCash(long price)
    {
	return cash >= price;
    }
}

class ShopItem extends Panel implements ActionListener
{
    private long price;
    private String name;
    private int order = 0;
    private Item item;
    private Label lPrice, lQuantity, lOrder;
    private Button add, remove;
    private ShopWindow owner;

    public ShopItem(ShopWindow owner, Item item)
    {
	this(item.getName(), item.getPrice(), item.getQuantityLabel());
	this.item = item;
	this.owner = owner;
    }

    public ShopItem(String name, long price, Label lQuantity)
    {
	//super(new FlowLayout(FlowLayout.RIGHT,0,0));
	super(new GridLayout(1,5));

	this.name = name;
	this.price = price;

	add(new Label(name, Label.CENTER));
	Panel t2 = new Panel(new FlowLayout(FlowLayout.CENTER,4,0));

	add = new Button("add");
	remove = new Button("remove");
	
	add.addActionListener(this);
	remove.addActionListener(this);

	t2.add(add);
	t2.add(remove);
	
	lOrder = new Label("0", Label.CENTER);
	lPrice = new Label(price+"", Label.CENTER);
	add(lPrice);
	add(lQuantity);
	add(lOrder);
	add(t2);
    }

    // make sure that only affordable items are enabled
    public void updateButtons()
    {
	add.setEnabled( price > 0 && owner.hasCash(price) );
	remove.setEnabled( price > 0 && order > 0 );
    }

    public void confirmOrder()
    {
	item.incQuantity(order);
    }
    
    public void actionPerformed(ActionEvent evt)
    {
	Object source = evt.getSource();

	if( source == add && add.isEnabled() )
	    {
		order++;
		owner.updateCash(-price);
		lOrder.setText(""+order);
		return;
	    }
	if( source == remove && remove.isEnabled() )
	    {
		order--;
		owner.updateCash(price);
		lOrder.setText(""+order);
		return;
	    }
    }
}
